@@ -31,7 +31,7 @@ The strings "abc", "ed" and "fgh" are atoms.
 
 When searching for regexps/hex strings matching a file, YARA uses these
 atoms to find locations inside the file where the regexp/hex string could
-match. If the atom "abc" is found somewhere inside the file, there is a change
+match. If the atom "abc" is found somewhere inside the file, there is a chance
 for /abc.*ed[0-9]+fgh/ to match the file, if "abc" doesn't appear in the file
 there's no chance for the regexp to match. When the atom is found in the file
 YARA proceeds to fully evaluate the regexp/hex string to determine if it's
@@ -41,7 +41,7 @@ For each regexp/hex string YARA extracts one or more atoms. Sometimes a
 single atom is enough (like in the previous example "abc" is enough for finding
 /abc.*ed[0-9]+fgh/), but sometimes a single atom isn't enough like in the
 regexp /(abc|efg)/. In this case YARA must search for both "abc" AND "efg" and
-fully evaluate the regexp whenever one of those atoms is found.
+fully evaluate the regexp whenever one of these atoms is found.
 
 In the regexp /Look(at|into)this/ YARA can search for "Look", or search for
 "this", or search for both "at" and "into". This is what we call an atoms tree,
@@ -89,10 +89,10 @@ will end up using the "Look" atom alone, but in /a(bcd|efg)h/ atoms "bcd" and
 //
 // Returns a numeric value indicating the quality of an atom. The quality
 // depends on some characteristics of the atom, including its length, number
-// of zeroes and number of unique distinct bytes. Atom 00 00 has a very low
-// quality, because it's only two bytes long and both bytes are zeroes. Atom
-// 01 01 01 01 is better but still not optimal, because the same byte is
-// repeated. Atom 01 02 03 04 is an optimal one.
+// of very common bytes like 00 and FF and number of unique distinct bytes.
+// Atom 00 00 has a very low quality, because it's only two bytes long and
+// both bytes are zeroes. Atom 01 01 01 01 is better but still not optimal,
+// because the same byte is repeated. Atom 01 02 03 04 is an optimal one.
 //
 // Args:
 //    uint8_t* atom   - Pointer to the atom's bytes.
@@ -106,15 +106,31 @@ int _yr_atoms_quality(
     uint8_t* atom,
     int atom_length)
 {
-  int null_bytes = 0;
+  int penalty = 0;
   int unique_bytes = 0;
   int is_unique;
   int i, j;
 
   for (i = 0; i < atom_length; i++)
   {
-    if (atom[i] == 0)
-      null_bytes++;
+    if (atom[i] == 0x00 || atom[i] == 0xFF)
+    {
+      // Penalize common bytes like 0x00 and 0xFF, specially if they are
+      // in the first two positions.
+
+      switch(i)
+      {
+        case 0:
+          penalty += 3;
+          break;
+        case 1:
+          penalty += 2;
+          break;
+        default:
+          penalty += 1;
+          break;
+      }
+    }
 
     is_unique = TRUE;
 
@@ -129,17 +145,17 @@ int _yr_atoms_quality(
       unique_bytes += 1;
   }
 
-  return atom_length + unique_bytes - null_bytes;
+  return atom_length + unique_bytes - penalty;
 }
 
 //
-// _yr_atoms_min_quality
+// yr_atoms_min_quality
 //
 // Returns the quality for the worst quality atom in a list.
 //
 
-int _yr_atoms_min_quality(
-  YR_ATOM_LIST_ITEM* atom_list)
+int yr_atoms_min_quality(
+    YR_ATOM_LIST_ITEM* atom_list)
 {
   YR_ATOM_LIST_ITEM* atom;
 
@@ -164,6 +180,7 @@ int _yr_atoms_min_quality(
   return min_quality;
 }
 
+
 //
 // _yr_atoms_tree_node_create
 //
@@ -171,7 +188,7 @@ int _yr_atoms_min_quality(
 //
 
 ATOM_TREE_NODE* _yr_atoms_tree_node_create(
-  uint8_t type)
+    uint8_t type)
 {
   ATOM_TREE_NODE* new_node;
 
@@ -329,7 +346,7 @@ int _yr_atoms_choose(
   {
   case ATOM_TREE_LEAF:
 
-    item = yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
+    item = (YR_ATOM_LIST_ITEM*) yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
 
     for (i = 0; i < node->atom_length; i++)
       item->atom[i] = node->atom[i];
@@ -502,7 +519,7 @@ int _yr_atoms_case_insentive(
 
     while (atom_length != 0)
     {
-      new_atom = yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
+      new_atom = (YR_ATOM_LIST_ITEM*) yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
 
       if (new_atom == NULL)
         return ERROR_INSUFICIENT_MEMORY;
@@ -552,7 +569,7 @@ int _yr_atoms_wide(
 
   while (atom != NULL)
   {
-    new_atom = yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
+    new_atom = (YR_ATOM_LIST_ITEM*) yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
 
     if (new_atom == NULL)
       return ERROR_INSUFICIENT_MEMORY;
@@ -811,7 +828,7 @@ int yr_atoms_extract_triplets(
 
       for (i = 0; i < 256; i++)
       {
-        atom = yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
+        atom = (YR_ATOM_LIST_ITEM*) yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
 
         if (atom == NULL)
           return ERROR_INSUFICIENT_MEMORY;
@@ -842,7 +859,7 @@ int yr_atoms_extract_triplets(
 
       for (i = 0; i < 16; i++)
       {
-        atom = yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
+        atom = (YR_ATOM_LIST_ITEM*) yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
 
         if (atom == NULL)
           return ERROR_INSUFICIENT_MEMORY;
@@ -877,7 +894,7 @@ int yr_atoms_extract_triplets(
 
       for (i = 0; i < 256; i++)
       {
-        atom = yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
+        atom = (YR_ATOM_LIST_ITEM*) yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
 
         if (atom == NULL)
           return ERROR_INSUFICIENT_MEMORY;
@@ -908,7 +925,7 @@ int yr_atoms_extract_triplets(
 
       for (i = 0; i < 16; i++)
       {
-        atom = yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
+        atom = (YR_ATOM_LIST_ITEM*) yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
 
         if (atom == NULL)
           return ERROR_INSUFICIENT_MEMORY;
@@ -948,7 +965,7 @@ int yr_atoms_extract_from_re(
     int flags,
     YR_ATOM_LIST_ITEM** atoms)
 {
-  ATOM_TREE* atom_tree = yr_malloc(sizeof(ATOM_TREE));
+  ATOM_TREE* atom_tree = (ATOM_TREE*) yr_malloc(sizeof(ATOM_TREE));
   ATOM_TREE_NODE* temp;
   YR_ATOM_LIST_ITEM* wide_atoms;
   YR_ATOM_LIST_ITEM* case_insentive_atoms;
@@ -988,7 +1005,7 @@ int yr_atoms_extract_from_re(
 
     yr_atoms_extract_triplets(re->root_node, &triplet_atoms);
 
-    if (min_atom_quality < _yr_atoms_min_quality(triplet_atoms))
+    if (min_atom_quality < yr_atoms_min_quality(triplet_atoms))
     {
       yr_atoms_list_destroy(*atoms);
       *atoms = triplet_atoms;
@@ -1047,7 +1064,7 @@ int yr_atoms_extract_from_string(
   int quality;
   int i, j, length;
 
-  item = yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
+  item = (YR_ATOM_LIST_ITEM*) yr_malloc(sizeof(YR_ATOM_LIST_ITEM));
 
   if (item == NULL)
     return ERROR_INSUFICIENT_MEMORY;
