@@ -14,6 +14,7 @@ See the License for the specific language governing permissions and
 limitations under the License.
 */
 
+#include <assert.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <string.h>
@@ -25,9 +26,10 @@ limitations under the License.
 #include <yara/mem.h>
 #include <yara/object.h>
 #include <yara/lexer.h>
+#include <yara/strutils.h>
 
 
-int yr_compiler_create(
+YR_API int yr_compiler_create(
     YR_COMPILER** compiler)
 {
   int result;
@@ -104,7 +106,7 @@ int yr_compiler_create(
 }
 
 
-void yr_compiler_destroy(
+YR_API void yr_compiler_destroy(
     YR_COMPILER* compiler)
 {
   int i;
@@ -154,7 +156,7 @@ void yr_compiler_destroy(
 }
 
 
-void yr_compiler_set_callback(
+YR_API void yr_compiler_set_callback(
     YR_COMPILER* compiler,
     YR_COMPILER_CALLBACK_FUNC callback)
 {
@@ -242,7 +244,7 @@ void _yr_compiler_pop_file_name(
 }
 
 
-char* yr_compiler_get_current_file_name(
+YR_API char* yr_compiler_get_current_file_name(
     YR_COMPILER* context)
 {
   if (context->file_name_stack_ptr > 0)
@@ -267,7 +269,7 @@ int _yr_compiler_set_namespace(
   int i;
   int found;
 
-  ns = yr_arena_base_address(compiler->namespaces_arena);
+  ns = (YR_NAMESPACE*) yr_arena_base_address(compiler->namespaces_arena);
   found = FALSE;
 
   for (i = 0; i < compiler->namespaces_count; i++)
@@ -278,7 +280,7 @@ int _yr_compiler_set_namespace(
       break;
     }
 
-    ns = yr_arena_next_address(
+    ns = (YR_NAMESPACE*) yr_arena_next_address(
         compiler->namespaces_arena,
         ns,
         sizeof(YR_NAMESPACE));
@@ -295,7 +297,7 @@ int _yr_compiler_set_namespace(
       result = yr_arena_allocate_struct(
           compiler->namespaces_arena,
           sizeof(YR_NAMESPACE),
-          (void*) &ns,
+          (void**) &ns,
           offsetof(YR_NAMESPACE, name),
           EOL);
 
@@ -314,7 +316,7 @@ int _yr_compiler_set_namespace(
   return ERROR_SUCCESS;
 }
 
-int yr_compiler_add_file(
+YR_API int yr_compiler_add_file(
     YR_COMPILER* compiler,
     FILE* rules_file,
     const char* namespace_,
@@ -337,7 +339,7 @@ int yr_compiler_add_file(
 }
 
 
-int yr_compiler_add_string(
+YR_API int yr_compiler_add_string(
     YR_COMPILER* compiler,
     const char* rules_string,
     const char* namespace_)
@@ -413,16 +415,16 @@ int _yr_compiler_compile_rules(
 
   if (result == ERROR_SUCCESS)
   {
-    rules_file_header->rules_list_head = yr_arena_base_address(
+    rules_file_header->rules_list_head = (YR_RULE*) yr_arena_base_address(
         compiler->rules_arena);
 
-    rules_file_header->externals_list_head = yr_arena_base_address(
-        compiler->externals_arena);
+    rules_file_header->externals_list_head = (YR_EXTERNAL_VARIABLE*) 
+		yr_arena_base_address(compiler->externals_arena);
 
-    rules_file_header->code_start = yr_arena_base_address(
+    rules_file_header->code_start = (uint8_t*) yr_arena_base_address(
         compiler->code_arena);
 
-    rules_file_header->automaton = yr_arena_base_address(
+    rules_file_header->automaton = (YR_AC_AUTOMATON*) yr_arena_base_address(
         compiler->automaton_arena);
   }
 
@@ -506,7 +508,7 @@ int _yr_compiler_compile_rules(
 }
 
 
-int yr_compiler_get_rules(
+YR_API int yr_compiler_get_rules(
     YR_COMPILER* compiler,
     YR_RULES** rules)
 {
@@ -518,7 +520,7 @@ int yr_compiler_get_rules(
   if (compiler->compiled_rules_arena == NULL)
      FAIL_ON_ERROR(_yr_compiler_compile_rules(compiler));
 
-  yara_rules = yr_malloc(sizeof(YR_RULES));
+  yara_rules = (YR_RULES*) yr_malloc(sizeof(YR_RULES));
 
   if (yara_rules == NULL)
     return ERROR_INSUFICIENT_MEMORY;
@@ -548,7 +550,7 @@ int yr_compiler_get_rules(
 }
 
 
-int yr_compiler_define_integer_variable(
+YR_API int yr_compiler_define_integer_variable(
     YR_COMPILER* compiler,
     const char* identifier,
     int64_t value)
@@ -592,7 +594,7 @@ int yr_compiler_define_integer_variable(
 }
 
 
-int yr_compiler_define_boolean_variable(
+YR_API int yr_compiler_define_boolean_variable(
     YR_COMPILER* compiler,
     const char* identifier,
     int value)
@@ -604,7 +606,7 @@ int yr_compiler_define_boolean_variable(
 }
 
 
-int yr_compiler_define_string_variable(
+YR_API int yr_compiler_define_string_variable(
     YR_COMPILER* compiler,
     const char* identifier,
     const char* value)
@@ -654,7 +656,7 @@ int yr_compiler_define_string_variable(
 }
 
 
-char* yr_compiler_get_error_message(
+YR_API char* yr_compiler_get_error_message(
     YR_COMPILER* compiler,
     char* buffer,
     int buffer_size)
@@ -664,39 +666,39 @@ char* yr_compiler_get_error_message(
     case ERROR_INSUFICIENT_MEMORY:
       snprintf(buffer, buffer_size, "not enough memory");
       break;
-    case ERROR_DUPLICATE_IDENTIFIER:
+    case ERROR_DUPLICATED_IDENTIFIER:
       snprintf(
           buffer,
           buffer_size,
-          "duplicate identifier \"%s\"",
+          "duplicated identifier \"%s\"",
           compiler->last_error_extra_info);
       break;
-    case ERROR_DUPLICATE_STRING_IDENTIFIER:
+    case ERROR_DUPLICATED_STRING_IDENTIFIER:
       snprintf(
           buffer,
           buffer_size,
-          "duplicate string identifier \"%s\"",
+          "duplicated string identifier \"%s\"",
           compiler->last_error_extra_info);
       break;
-    case ERROR_DUPLICATE_TAG_IDENTIFIER:
+    case ERROR_DUPLICATED_TAG_IDENTIFIER:
       snprintf(
           buffer,
           buffer_size,
-          "duplicate tag identifier \"%s\"",
+          "duplicated tag identifier \"%s\"",
           compiler->last_error_extra_info);
       break;
-    case ERROR_DUPLICATE_META_IDENTIFIER:
+    case ERROR_DUPLICATED_META_IDENTIFIER:
       snprintf(
           buffer,
           buffer_size,
-          "duplicate metadata identifier \"%s\"",
+          "duplicated metadata identifier \"%s\"",
           compiler->last_error_extra_info);
       break;
-    case ERROR_DUPLICATE_LOOP_IDENTIFIER:
+    case ERROR_DUPLICATED_LOOP_IDENTIFIER:
       snprintf(
           buffer,
           buffer_size,
-          "duplicate loop identifier \"%s\"",
+          "duplicated loop identifier \"%s\"",
           compiler->last_error_extra_info);
       break;
     case ERROR_UNDEFINED_STRING:
@@ -727,11 +729,11 @@ char* yr_compiler_get_error_message(
           "\"%s\" is not a structure",
           compiler->last_error_extra_info);
       break;
-    case ERROR_NOT_AN_ARRAY:
+    case ERROR_NOT_INDEXABLE:
       snprintf(
           buffer,
           buffer_size,
-          "\"%s\" is not a array",
+          "\"%s\" is not an array or dictionary",
           compiler->last_error_extra_info);
       break;
     case ERROR_INVALID_FIELD_NAME:
@@ -775,11 +777,27 @@ char* yr_compiler_get_error_message(
           "unknown module \"%s\"",
           compiler->last_error_extra_info);
       break;
+    case ERROR_DUPLICATED_STRUCTURE_MEMBER:
+      snprintf(buffer,
+          buffer_size,
+          "duplicated structure member");
+      break;
+    case ERROR_WRONG_ARGUMENTS:
+      snprintf(
+          buffer,
+          buffer_size,
+          "wrong arguments for function \"%s\"",
+          compiler->last_error_extra_info);
+      break;
+    case ERROR_WRONG_RETURN_TYPE:
+      snprintf(buffer,
+          buffer_size,
+          "wrong return type for overloaded function");
+      break;
     case ERROR_INVALID_HEX_STRING:
     case ERROR_INVALID_REGULAR_EXPRESSION:
     case ERROR_SYNTAX_ERROR:
     case ERROR_WRONG_TYPE:
-    case ERROR_WRONG_NUMBER_OF_ARGUMENTS:
       snprintf(
           buffer,
           buffer_size,
