@@ -14,6 +14,8 @@ See the License for the specific language governing permissions and
 limitations under the License.
 */
 
+#define _GNU_SOURCE
+
 #include <string.h>
 #include <assert.h>
 #include <time.h>
@@ -24,10 +26,12 @@ limitations under the License.
 #include <yara/object.h>
 #include <yara/modules.h>
 #include <yara/re.h>
-
+#include <yara/strutils.h>
+#include <yara/utils.h>
 
 #include <yara.h>
 
+
 #define STACK_SIZE 16384
 #define MEM_SIZE   MAX_LOOP_NESTING * LOOP_LOCAL_VARS
 
@@ -42,16 +46,32 @@ limitations under the License.
 #define pop(x)  x = stack[--sp]
 
 
-#define operation(operator, op1, op2) \
-    (IS_UNDEFINED(op1) || IS_UNDEFINED(op2)) ? (UNDEFINED) : (op1 operator op2)
+#define little_endian_uint8_t(x)     (x)
+#define little_endian_uint16_t(x)    (x)
+#define little_endian_uint32_t(x)    (x)
+#define little_endian_int8_t(x)      (x)
+#define little_endian_int16_t(x)     (x)
+#define little_endian_int32_t(x)     (x)
+
+#define big_endian_uint8_t(x)         (x)
+
+#define big_endian_uint16_t(x) \
+    (((((uint16_t)(x) & 0xFF)) << 8) | \
+     ((((uint16_t)(x) & 0xFF00)) >> 8))
+
+#define big_endian_uint32_t(x) \
+    (((((uint32_t)(x) & 0xFF)) << 24) | \
+     ((((uint32_t)(x) & 0xFF00)) << 8) | \
+     ((((uint32_t)(x) & 0xFF0000)) >> 8) | \
+     ((((uint32_t)(x) & 0xFF000000)) >> 24))
+
+#define big_endian_int8_t(x)   big_endian_uint8_t(x)
+#define big_endian_int16_t(x)  big_endian_uint16_t(x)
+#define big_endian_int32_t(x)  big_endian_uint32_t(x)
 
 
-#define comparison(operator, op1, op2) \
-    (IS_UNDEFINED(op1) || IS_UNDEFINED(op2)) ? (0) : (op1 operator op2)
-
-
-#define function_read(type) \
-    int64_t read_##type(YR_MEMORY_BLOCK* block, size_t offset) \
+#define function_read(type, endianess) \
+    int64_t read_##type##_##endianess(YR_MEMORY_BLOCK* block, size_t offset) \
     { \
       while (block != NULL) \
       { \
@@ -59,19 +79,28 @@ limitations under the License.
             block->size >= sizeof(type) && \
             offset <= block->base + block->size - sizeof(type)) \
         { \
-          return *((type *) (block->data + offset - block->base)); \
+          type result = *(type *)(block->data + offset - block->base); \
+          result = endianess##_##type(result); \
+          return result; \
         } \
         block = block->next; \
       } \
       return UNDEFINED; \
     };
 
-function_read(uint8_t)
-function_read(uint16_t)
-function_read(uint32_t)
-function_read(int8_t)
-function_read(int16_t)
-function_read(int32_t)
+
+function_read(uint8_t, little_endian)
+function_read(uint16_t, little_endian)
+function_read(uint32_t, little_endian)
+function_read(int8_t, little_endian)
+function_read(int16_t, little_endian)
+function_read(int32_t, little_endian)
+function_read(uint8_t, big_endian)
+function_read(uint16_t, big_endian)
+function_read(uint32_t, big_endian)
+function_read(int8_t, big_endian)
+function_read(int16_t, big_endian)
+function_read(int32_t, big_endian)
 
 
 int yr_execute_code(
@@ -95,7 +124,11 @@ int yr_execute_code(
   YR_OBJECT* object;
   YR_OBJECT_FUNCTION* function;
 
+  SIZED_STRING* sized_str_1;
+  SIZED_STRING* sized_str_2;
+
   char* identifier;
+  char* args_fmt;
 
   int i;
   int found;
@@ -138,7 +171,8 @@ int yr_execute_code(
         r1 = *(uint64_t*)(ip + 1);
         ip += sizeof(uint64_t);
         pop(r2);
-        mem[r1] += r2;
+        if (!IS_UNDEFINED(r2))
+          mem[r1] += r2;
         break;
 
       case OP_INCR_M:
@@ -211,7 +245,7 @@ int yr_execute_code(
         if (IS_UNDEFINED(r1) || IS_UNDEFINED(r2))
           push(0);
         else
-          push(r1 & r2);
+          push(r1 && r2);
         break;
 
       case OP_OR:
@@ -222,7 +256,7 @@ int yr_execute_code(
         else if (IS_UNDEFINED(r2))
           push(r1);
         else
-          push(r1 | r2);
+          push(r1 || r2);
         break;
 
       case OP_NOT:
@@ -236,122 +270,164 @@ int yr_execute_code(
       case OP_LT:
         pop(r2);
         pop(r1);
-        push(comparison(<, r1, r2));
+        push(COMPARISON(<, r1, r2));
         break;
 
       case OP_GT:
         pop(r2);
         pop(r1);
-        push(comparison(>, r1, r2));
+        push(COMPARISON(>, r1, r2));
         break;
 
       case OP_LE:
         pop(r2);
         pop(r1);
-        push(comparison(<=, r1, r2));
+        push(COMPARISON(<=, r1, r2));
         break;
 
       case OP_GE:
         pop(r2);
         pop(r1);
-        push(comparison(>=, r1, r2));
+        push(COMPARISON(>=, r1, r2));
         break;
 
       case OP_EQ:
         pop(r2);
         pop(r1);
-        push(comparison(==, r1, r2));
+        push(COMPARISON(==, r1, r2));
         break;
 
       case OP_NEQ:
         pop(r2);
         pop(r1);
-        push(comparison(!=, r1, r2));
+        push(COMPARISON(!=, r1, r2));
         break;
 
-      case OP_SZ_EQ:
+      case OP_STR_EQ:
         pop(r2);
         pop(r1);
 
         if (IS_UNDEFINED(r1) || IS_UNDEFINED(r2))
+        {
           push(UNDEFINED);
+        }
         else
-          push(strcmp(UINT64_TO_PTR(char*, r1),
-                      UINT64_TO_PTR(char*, r2)) == 0);
+        {
+          sized_str_1 = UINT64_TO_PTR(SIZED_STRING*, r1);
+          sized_str_2 = UINT64_TO_PTR(SIZED_STRING*, r2);
+
+          if (sized_str_1->length == sized_str_2->length)
+          {
+            push(memcmp(sized_str_1->c_string,
+                        sized_str_2->c_string,
+                        sized_str_2->length) == 0);
+          }
+          else
+          {
+            push(FALSE);
+          }
+        }
         break;
 
-      case OP_SZ_NEQ:
+      case OP_STR_NEQ:
         pop(r2);
         pop(r1);
 
         if (IS_UNDEFINED(r1) || IS_UNDEFINED(r2))
+        {
           push(UNDEFINED);
+        }
         else
-          push(strcmp(UINT64_TO_PTR(char*, r1),
-                      UINT64_TO_PTR(char*, r2)) != 0);
+        {
+          sized_str_1 = UINT64_TO_PTR(SIZED_STRING*, r1);
+          sized_str_2 = UINT64_TO_PTR(SIZED_STRING*, r2);
+
+          if (sized_str_1->length == sized_str_2->length)
+          {
+            push(memcmp(sized_str_1->c_string,
+                        sized_str_2->c_string,
+                        sized_str_2->length) != 0);
+          }
+          else
+          {
+            push(TRUE);
+          }
+        }
         break;
 
-      case OP_SZ_TO_BOOL:
+      case OP_STR_TO_BOOL:
         pop(r1);
 
         if (IS_UNDEFINED(r1))
           push(UNDEFINED);
         else
-          push(strlen(UINT64_TO_PTR(char*, r1)) > 0);
+          push(UINT64_TO_PTR(SIZED_STRING*, r1)->length > 0);
 
         break;
 
       case OP_ADD:
         pop(r2);
         pop(r1);
-        push(operation(+, r1, r2));
+        push(OPERATION(+, r1, r2));
         break;
 
       case OP_SUB:
         pop(r2);
         pop(r1);
-        push(operation(-, r1, r2));
+        push(OPERATION(-, r1, r2));
         break;
 
       case OP_MUL:
         pop(r2);
         pop(r1);
-        push(operation(*, r1, r2));
+        push(OPERATION(*, r1, r2));
         break;
 
       case OP_DIV:
         pop(r2);
         pop(r1);
-        push(operation(/, r1, r2));
+        push(OPERATION(/, r1, r2));
         break;
 
       case OP_MOD:
         pop(r2);
         pop(r1);
-        push(operation(%, r1, r2));
-        break;
-
-      case OP_NEG:
-        pop(r1);
-        push(IS_UNDEFINED(r1) ? UNDEFINED : ~r1);
+        push(OPERATION(%, r1, r2));
         break;
 
       case OP_SHR:
         pop(r2);
         pop(r1);
-        push(operation(>>, r1, r2));
+        push(OPERATION(>>, r1, r2));
         break;
 
       case OP_SHL:
         pop(r2);
         pop(r1);
-        push(operation(<<, r1, r2));
+        push(OPERATION(<<, r1, r2));
         break;
 
-      case OP_XOR:
+      case OP_BITWISE_NOT:
+        pop(r1);
+        push(IS_UNDEFINED(r1) ? UNDEFINED : ~r1);
+        break;
+
+      case OP_BITWISE_AND:
         pop(r2);
         pop(r1);
-        push(operation(^, r1, r2));
+        push(OPERATION(&, r1, r2));
+        break;
+
+      case OP_BITWISE_OR:
+        pop(r2);
+        pop(r1);
+        push(OPERATION(|, r1, r2));
+        break;
+
+      case OP_BITWISE_XOR:
+        pop(r2);
+        pop(r1);
+        push(OPERATION(^, r1, r2));
         break;
 
       case OP_PUSH_RULE:
@@ -436,10 +512,10 @@ int yr_execute_code(
         break;
 
       case OP_INDEX_ARRAY:
-        pop(r1);
-        pop(r2);
+        pop(r1);  // index
+        pop(r2);  // array
 
-        if (r1 == UNDEFINED)
+        if (IS_UNDEFINED(r1))
         {
           push(UNDEFINED);
           break;
@@ -456,25 +532,71 @@ int yr_execute_code(
 
         break;
 
+      case OP_LOOKUP_DICT:
+        pop(r1);  // key
+        pop(r2);  // dictionary
+
+        if (IS_UNDEFINED(r1))
+        {
+          push(UNDEFINED);
+          break;
+        }
+
+        object = UINT64_TO_PTR(YR_OBJECT*, r2);
+        assert(object->type == OBJECT_TYPE_DICTIONARY);
+
+        object = yr_object_dict_get_item(
+            object, 0, UINT64_TO_PTR(SIZED_STRING*, r1)->c_string);
+
+        if (object != NULL)
+          push(PTR_TO_UINT64(object));
+        else
+          push(UNDEFINED);
+
+        break;
+
       case OP_CALL:
-
-        // r1 = number of arguments
-
-        r1 = *(uint64_t*)(ip + 1);
+        args_fmt = *(char**)(ip + 1);
         ip += sizeof(uint64_t);
 
+        i = strlen(args_fmt);
+
         // pop arguments from stack and copy them to args array
 
-        while (r1 > 0)
+        while (i > 0)
         {
-          pop(args[r1 - 1]);
-          r1--;
+          pop(args[i - 1]);
+          i--;
         }
 
         pop(r2);
 
+        if (IS_UNDEFINED(r2))
+        {
+          push(UNDEFINED);
+          break;
+        }
+
         function = UINT64_TO_PTR(YR_OBJECT_FUNCTION*, r2);
-        result = function->code((void*) args, context, function);
+        result = ERROR_INTERNAL_FATAL_ERROR;
+
+        for (i = 0; i < MAX_OVERLOADED_FUNCTIONS; i++)
+        {
+          if (function->prototypes[i].arguments_fmt == NULL)
+            break;
+
+          if (strcmp(function->prototypes[i].arguments_fmt, args_fmt) == 0)
+          {
+            result = function->prototypes[i].code(
+                (void*) args,
+                context,
+                function);
+
+            break;
+          }
+        }
+
+        assert(i < MAX_OVERLOADED_FUNCTIONS);
 
         if (result == ERROR_SUCCESS)
           push(PTR_TO_UINT64(function->return_obj));
@@ -483,13 +605,13 @@ int yr_execute_code(
 
         break;
 
-      case OP_STR_FOUND:
+      case OP_FOUND:
         pop(r1);
         string = UINT64_TO_PTR(YR_STRING*, r1);
         push(string->matches[tidx].tail != NULL ? 1 : 0);
         break;
 
-      case OP_STR_FOUND_AT:
+      case OP_FOUND_AT:
         pop(r2);
         pop(r1);
 
@@ -523,14 +645,14 @@ int yr_execute_code(
 
         break;
 
-      case OP_STR_FOUND_IN:
+      case OP_FOUND_IN:
         pop(r3);
         pop(r2);
         pop(r1);
 
         if (IS_UNDEFINED(r1) || IS_UNDEFINED(r2))
         {
-          push(0);
+          push(UNDEFINED);
           break;
         }
 
@@ -558,13 +680,13 @@ int yr_execute_code(
 
         break;
 
-      case OP_STR_COUNT:
+      case OP_COUNT:
         pop(r1);
         string = UINT64_TO_PTR(YR_STRING*, r1);
         push(string->matches[tidx].count);
         break;
 
-      case OP_STR_OFFSET:
+      case OP_OFFSET:
         pop(r2);
         pop(r1);
 
@@ -629,39 +751,80 @@ int yr_execute_code(
 
       case OP_INT8:
         pop(r1);
-        push(read_int8_t(context->mem_block, r1));
+        push(read_int8_t_little_endian(context->mem_block, r1));
         break;
 
       case OP_INT16:
         pop(r1);
-        push(read_int16_t(context->mem_block, r1));
+        push(read_int16_t_little_endian(context->mem_block, r1));
         break;
 
       case OP_INT32:
         pop(r1);
-        push(read_int32_t(context->mem_block, r1));
+        push(read_int32_t_little_endian(context->mem_block, r1));
         break;
 
       case OP_UINT8:
         pop(r1);
-        push(read_uint8_t(context->mem_block, r1));
+        push(read_uint8_t_little_endian(context->mem_block, r1));
         break;
 
       case OP_UINT16:
         pop(r1);
-        push(read_uint16_t(context->mem_block, r1));
+        push(read_uint16_t_little_endian(context->mem_block, r1));
         break;
 
       case OP_UINT32:
         pop(r1);
-        push(read_uint32_t(context->mem_block, r1));
+        push(read_uint32_t_little_endian(context->mem_block, r1));
+        break;
+
+      case OP_INT8BE:
+        pop(r1);
+        push(read_int8_t_big_endian(context->mem_block, r1));
+        break;
+
+      case OP_INT16BE:
+        pop(r1);
+        push(read_int16_t_big_endian(context->mem_block, r1));
+        break;
+
+      case OP_INT32BE:
+        pop(r1);
+        push(read_int32_t_big_endian(context->mem_block, r1));
+        break;
+
+      case OP_UINT8BE:
+        pop(r1);
+        push(read_uint8_t_big_endian(context->mem_block, r1));
+        break;
+
+      case OP_UINT16BE:
+        pop(r1);
+        push(read_uint16_t_big_endian(context->mem_block, r1));
+        break;
+
+      case OP_UINT32BE:
+        pop(r1);
+        push(read_uint32_t_big_endian(context->mem_block, r1));
         break;
 
       case OP_CONTAINS:
         pop(r2);
         pop(r1);
-        push(strstr(UINT64_TO_PTR(char*, r1),
-                    UINT64_TO_PTR(char*, r2)) != NULL);
+
+        if (IS_UNDEFINED(r1) || IS_UNDEFINED(r2))
+        {
+          push(UNDEFINED);
+        }
+        else
+        {
+          sized_str_1 = UINT64_TO_PTR(SIZED_STRING*, r1);
+          sized_str_2 = UINT64_TO_PTR(SIZED_STRING*, r2);
+
+          push(memmem(sized_str_1->c_string, sized_str_1->length,
+                      sized_str_2->c_string, sized_str_2->length) != NULL);
+        }
         break;
 
       case OP_IMPORT:
@@ -678,9 +841,9 @@ int yr_execute_code(
         pop(r2);
         pop(r1);
 
-        count = strlen(UINT64_TO_PTR(char*, r1));
+        sized_str_1 = UINT64_TO_PTR(SIZED_STRING*, r1);
 
-        if (count == 0)
+        if (sized_str_1->length == 0)
         {
           push(FALSE);
           break;
@@ -688,8 +851,8 @@ int yr_execute_code(
 
         result = yr_re_exec(
           UINT64_TO_PTR(uint8_t*, r2),
-          UINT64_TO_PTR(uint8_t*, r1),
-          count,
+          (uint8_t*) sized_str_1->c_string,
+          sized_str_1->length,
           RE_FLAGS_SCAN,
           NULL,
           NULL);
