@@ -17,11 +17,6 @@ limitations under the License.
 #ifndef YR_YARA_H
 #define YR_YARA_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-
 #include "yara/utils.h"
 #include "yara/filemap.h"
 #include "yara/compiler.h"
@@ -30,8 +25,4 @@ extern "C" {
 #include "yara/libyara.h"
 #include "yara/error.h"
 
-#ifdef __cplusplus
-}  /* end extern "C" */
-#endif
-
 #endif
