@@ -27,11 +27,16 @@ limitations under the License.
 #define yr_realloc realloc
 #define yr_free free
 #define yr_strdup strdup
+#define yr_strndup strndup
 
 #include <dmalloc.h>
 
 #else
 
+void* yr_calloc(
+    size_t count,
+    size_t size);
+
 void* yr_malloc(
     size_t size);
 
@@ -45,6 +50,9 @@ void yr_free(
 char* yr_strdup(
     const char *str);
 
+char* yr_strndup(
+	const char *str, size_t n);
+
 #endif
 
 int yr_heap_alloc();
