@@ -45,6 +45,12 @@ int yr_heap_free()
 }
 
 
+void* yr_calloc(size_t count, size_t size)
+{
+  return (void*) HeapAlloc(hHeap, HEAP_ZERO_MEMORY, count * size);
+}
+
+
 void* yr_malloc(size_t size)
 {
   return (void*) HeapAlloc(hHeap, HEAP_ZERO_MEMORY, size);
@@ -65,15 +71,33 @@ void yr_free(void* ptr)
 
 char* yr_strdup(const char *str)
 {
-  size_t len = strlen(str) + 1;
-  void *dup = yr_malloc(len);
+  size_t len = strlen(str);
+  char *dup = (char*) yr_malloc(len + 1);
 
-  if (dup != NULL)
-    memcpy(dup, str, len);
+  if (dup == NULL)
+    return NULL;
+
+  memcpy(dup, str, len);
+  dup[len] = '\0';
 
   return (char*) dup;
 }
 
+
+char* yr_strndup(const char *str, size_t n)
+{
+  size_t len = strnlen(str, n);
+  char *dup = (char*) yr_malloc(len + 1);
+
+  if (dup == NULL)
+    return NULL;
+
+  memcpy(dup, str, len);
+  dup[len] = '\0';
+
+  return (char *) dup;
+}
+
 #else
 
 #include <stdlib.h>
@@ -94,6 +118,12 @@ int yr_heap_free()
 }
 
 
+void* yr_calloc(size_t count, size_t size)
+{
+  return calloc(count, size);
+}
+
+
 void* yr_malloc(size_t size)
 {
   return malloc(size);
@@ -117,4 +147,10 @@ char* yr_strdup(const char *str)
   return strdup(str);
 }
 
+
+char* yr_strndup(const char *str, size_t n)
+{
+  return strndup(str, n);
+}
+
 #endif
