@@ -21,6 +21,11 @@ limitations under the License.
 #include <stdlib.h>
 #include <string.h>
 
+#if _WIN32
+#define PRIu64 "%I64d"
+#else
+#include <inttypes.h>
+#endif
 
 #include <yara/mem.h>
 #include <yara/error.h>
@@ -46,6 +51,9 @@ int yr_object_create(
     case OBJECT_TYPE_ARRAY:
       object_size = sizeof(YR_OBJECT_ARRAY);
       break;
+    case OBJECT_TYPE_DICTIONARY:
+      object_size = sizeof(YR_OBJECT_DICTIONARY);
+      break;
     case OBJECT_TYPE_INTEGER:
       object_size = sizeof(YR_OBJECT_INTEGER);
       break;
@@ -85,6 +93,11 @@ int yr_object_create(
       break;
     case OBJECT_TYPE_ARRAY:
       ((YR_OBJECT_ARRAY*) obj)->items = NULL;
+      ((YR_OBJECT_ARRAY*) obj)->prototype_item = NULL;
+      break;
+    case OBJECT_TYPE_DICTIONARY:
+      ((YR_OBJECT_DICTIONARY*) obj)->items = NULL;
+      ((YR_OBJECT_ARRAY*) obj)->prototype_item = NULL;
       break;
     case OBJECT_TYPE_INTEGER:
       ((YR_OBJECT_INTEGER*) obj)->value = UNDEFINED;
@@ -96,16 +109,21 @@ int yr_object_create(
       ((YR_OBJECT_REGEXP*) obj)->value = NULL;
       break;
     case OBJECT_TYPE_FUNCTION:
-      ((YR_OBJECT_FUNCTION*) obj)->arguments_fmt = NULL;
       ((YR_OBJECT_FUNCTION*) obj)->return_obj = NULL;
+      for (int i = 0; i < MAX_OVERLOADED_FUNCTIONS; i++)
+      {
+        ((YR_OBJECT_FUNCTION*) obj)->prototypes[i].arguments_fmt = NULL;
+        ((YR_OBJECT_FUNCTION*) obj)->prototypes[i].code = NULL;
+      }
       break;
   }
 
   if (parent != NULL)
   {
-    assert( parent->type == OBJECT_TYPE_STRUCTURE ||
-            parent->type == OBJECT_TYPE_ARRAY ||
-            parent->type == OBJECT_TYPE_FUNCTION);
+    assert(parent->type == OBJECT_TYPE_STRUCTURE ||
+           parent->type == OBJECT_TYPE_ARRAY ||
+           parent->type == OBJECT_TYPE_DICTIONARY ||
+           parent->type == OBJECT_TYPE_FUNCTION);
 
     switch(parent->type)
     {
@@ -116,9 +134,11 @@ int yr_object_create(
         break;
 
       case OBJECT_TYPE_ARRAY:
-        FAIL_ON_ERROR_WITH_CLEANUP(
-            yr_object_array_set_item(parent, obj, 0),
-            yr_free(obj));
+        ((YR_OBJECT_ARRAY*) parent)->prototype_item = obj;
+        break;
+
+      case OBJECT_TYPE_DICTIONARY:
+        ((YR_OBJECT_DICTIONARY*) parent)->prototype_item = obj;
         break;
     }
   }
@@ -139,9 +159,10 @@ int yr_object_function_create(
     YR_OBJECT** function)
 {
   YR_OBJECT* return_obj;
-  YR_OBJECT* f;
+  YR_OBJECT* f = NULL;
 
   int8_t return_type;
+  int i;
 
   switch (*return_fmt)
   {
@@ -155,19 +176,46 @@ int yr_object_function_create(
       return ERROR_INVALID_FORMAT;
   }
 
-  FAIL_ON_ERROR(yr_object_create(
-      OBJECT_TYPE_FUNCTION,
-      identifier,
-      parent,
-      &f));
+  if (parent != NULL)
+  {
+    assert(parent->type == OBJECT_TYPE_STRUCTURE);
 
-  FAIL_ON_ERROR_WITH_CLEANUP(
-      yr_object_create(return_type, "result", f, &return_obj),
-      yr_object_destroy(f));
+    // Try to find if the structure already has a function
+    // with that name. In that case this is a function oveload.
 
-  ((YR_OBJECT_FUNCTION* )f)->arguments_fmt = arguments_fmt;
-  ((YR_OBJECT_FUNCTION* )f)->return_obj = return_obj;
-  ((YR_OBJECT_FUNCTION* )f)->code = code;
+    f = yr_object_lookup_field(parent, identifier);
+
+    if (f != NULL && return_type != ((YR_OBJECT_FUNCTION*) f)->return_obj->type)
+      return ERROR_WRONG_RETURN_TYPE;
+  }
+
+  if (f == NULL)
+  {
+    // Function doesn't exist yet, create it.
+
+    FAIL_ON_ERROR(yr_object_create(
+        OBJECT_TYPE_FUNCTION,
+        identifier,
+        parent,
+        &f));
+
+    FAIL_ON_ERROR_WITH_CLEANUP(
+        yr_object_create(return_type, "result", f, &return_obj),
+        yr_object_destroy(f));
+
+    ((YR_OBJECT_FUNCTION*) f)->return_obj = return_obj;
+  }
+
+  for (i = 0; i < MAX_OVERLOADED_FUNCTIONS; i++)
+  {
+    if (((YR_OBJECT_FUNCTION*) f)->prototypes[i].arguments_fmt == NULL)
+    {
+      ((YR_OBJECT_FUNCTION*) f)->prototypes[i].arguments_fmt = arguments_fmt;
+      ((YR_OBJECT_FUNCTION*) f)->prototypes[i].code = code;
+
+      break;
+    }
+  }
 
   if (function != NULL)
     *function = f;
@@ -212,12 +260,14 @@ int yr_object_from_external_variable(
     {
       case EXTERNAL_VARIABLE_TYPE_INTEGER:
       case EXTERNAL_VARIABLE_TYPE_BOOLEAN:
-        yr_object_set_integer(external->integer, obj, NULL);
+        yr_object_set_integer(
+            external->integer, obj, NULL);
         break;
 
       case EXTERNAL_VARIABLE_TYPE_STRING:
       case EXTERNAL_VARIABLE_TYPE_MALLOC_STRING:
-        yr_object_set_string(external->string, obj, NULL);
+        yr_object_set_string(
+            external->string, strlen(external->string), obj, NULL);
         break;
     }
 
@@ -234,10 +284,11 @@ void yr_object_destroy(
   YR_STRUCTURE_MEMBER* member;
   YR_STRUCTURE_MEMBER* next_member;
   YR_ARRAY_ITEMS* array_items;
+  YR_DICTIONARY_ITEMS* dict_items;
 
   RE* re;
   int i;
-  char* str;
+  SIZED_STRING* str;
 
   switch(object->type)
   {
@@ -266,15 +317,42 @@ void yr_object_destroy(
       break;
 
     case OBJECT_TYPE_ARRAY:
+      if (((YR_OBJECT_ARRAY*) object)->prototype_item != NULL)
+        yr_free(((YR_OBJECT_ARRAY*) object)->prototype_item);
+
       array_items = ((YR_OBJECT_ARRAY*) object)->items;
 
-      for (i = 0; i < array_items->count; i++)
-        if (array_items->objects[i] != NULL)
-          yr_object_destroy(array_items->objects[i]);
+      if (array_items != NULL)
+      {
+        for (i = 0; i < array_items->count; i++)
+          if (array_items->objects[i] != NULL)
+            yr_object_destroy(array_items->objects[i]);
+      }
 
       yr_free(array_items);
       break;
 
+    case OBJECT_TYPE_DICTIONARY:
+      if (((YR_OBJECT_DICTIONARY*) object)->prototype_item != NULL)
+        yr_free(((YR_OBJECT_DICTIONARY*) object)->prototype_item);
+
+      dict_items = ((YR_OBJECT_DICTIONARY*) object)->items;
+
+      if (dict_items != NULL)
+      {
+        for (i = 0; i < dict_items->used; i++)
+        {
+          if (dict_items->objects[i].key != NULL)
+            yr_free(dict_items->objects[i].key);
+
+          if (dict_items->objects[i].obj != NULL)
+            yr_object_destroy(dict_items->objects[i].obj);
+        }
+      }
+
+      yr_free(dict_items);
+      break;
+
     case OBJECT_TYPE_FUNCTION:
       yr_object_destroy(((YR_OBJECT_FUNCTION*) object)->return_obj);
       break;
@@ -317,26 +395,28 @@ YR_OBJECT* _yr_object_lookup(
   YR_OBJECT* obj = object;
 
   const char* p = pattern;
-  char field_name[256];
+  const char* key = NULL;
+
+  char str[256];
 
   int i;
-  int index;
+  int index = -1;
 
   while (obj != NULL)
   {
     i = 0;
 
-    while(*p != '\0' && *p != '.' && *p != '[')
+    while(*p != '\0' && *p != '.' && *p != '[' && i < sizeof(str))
     {
-      field_name[i++] = *p++;
+      str[i++] = *p++;
     }
 
-    field_name[i] = '\0';
+    str[i] = '\0';
 
     if (obj->type != OBJECT_TYPE_STRUCTURE)
       return NULL;
 
-    obj = yr_object_lookup_field(obj, field_name);
+    obj = yr_object_lookup_field(obj, str);
 
     if (obj == NULL)
       return NULL;
@@ -347,20 +427,37 @@ YR_OBJECT* _yr_object_lookup(
 
       if (*p == '%')
       {
-        if (*++p == 'i')
+        p++;
+
+        switch(*p++)
         {
-          index = va_arg(args, int);
-          p++;
-        }
-        else
-        {
-          return NULL;
+          case 'i':
+            index = va_arg(args, int);
+            break;
+          case 's':
+            key = va_arg(args, const char*);
+            break;
+
+          default:
+            return NULL;
         }
       }
       else if (*p >= '0' && *p <= '9')
       {
         index = strtol(p, (char**) &p, 10);
       }
+      else if (*p == '"')
+      {
+        i = 0;
+        p++;              // skip the opening quotation mark
+
+        while (*p != '"' && *p != '\0' && i < sizeof(str))
+          str[i++] = *p++;
+
+        str[i] = '\0';
+        p++;              // skip the closing quotation mark
+        key = str;
+      }
       else
       {
         return NULL;
@@ -369,7 +466,18 @@ YR_OBJECT* _yr_object_lookup(
       assert(*p++ == ']');
       assert(*p == '.' || *p == '\0');
 
-      obj = yr_object_array_get_item(obj, flags, index);
+      switch(obj->type)
+      {
+        case OBJECT_TYPE_ARRAY:
+          assert(index != -1);
+          obj = yr_object_array_get_item(obj, flags, index);
+          break;
+
+        case OBJECT_TYPE_DICTIONARY:
+          assert(key != NULL);
+          obj = yr_object_dict_get_item(obj, flags, key);
+          break;
+      }
     }
 
     if (*p == '\0')
@@ -411,6 +519,8 @@ int yr_object_copy(
 
   YR_ARRAY_ITEMS* array_items;
   YR_STRUCTURE_MEMBER* structure_member;
+  YR_OBJECT_FUNCTION* func;
+  YR_OBJECT_FUNCTION* func_copy;
 
   int i;
 
@@ -436,6 +546,20 @@ int yr_object_copy(
       ((YR_OBJECT_REGEXP*) copy)->value = NULL;
       break;
 
+    case OBJECT_TYPE_FUNCTION:
+
+      func = (YR_OBJECT_FUNCTION*) object;
+      func_copy = (YR_OBJECT_FUNCTION*) copy;
+
+      FAIL_ON_ERROR_WITH_CLEANUP(
+        yr_object_copy(func->return_obj, &func_copy->return_obj),
+        yr_object_destroy(copy));
+
+      for (i = 0; i < MAX_OVERLOADED_FUNCTIONS; i++)
+        func_copy->prototypes[i] = func->prototypes[i];
+
+      break;
+
     case OBJECT_TYPE_STRUCTURE:
 
       structure_member = ((YR_OBJECT_STRUCTURE*) object)->members;
@@ -492,7 +616,12 @@ int yr_object_structure_set_member(
 
   assert(object->type == OBJECT_TYPE_STRUCTURE);
 
-  sm = yr_malloc(sizeof(YR_STRUCTURE_MEMBER));
+  // Check if the object already have a member with the same identifier
+
+  if (yr_object_lookup_field(object,  member->identifier) != NULL)
+    return ERROR_DUPLICATED_STRUCTURE_MEMBER;
+
+  sm = (YR_STRUCTURE_MEMBER*) yr_malloc(sizeof(YR_STRUCTURE_MEMBER));
 
   if (sm == NULL)
     return ERROR_INSUFICIENT_MEMORY;
@@ -513,20 +642,18 @@ YR_OBJECT* yr_object_array_get_item(
     int index)
 {
   YR_OBJECT* result = NULL;
-  YR_ARRAY_ITEMS* array_items;
+  YR_OBJECT_ARRAY* array;
 
   assert(object->type == OBJECT_TYPE_ARRAY);
 
-  array_items = ((YR_OBJECT_ARRAY*) object)->items;
+  array = (YR_OBJECT_ARRAY*) object;
 
-  assert(array_items != NULL);
-
-  if (array_items->count > index)
-    result = array_items->objects[index];
+  if (array->items != NULL && array->items->count > index)
+      result = array->items->objects[index];
 
   if (result == NULL && flags & OBJECT_CREATE)
   {
-    yr_object_copy(array_items->objects[0], &result);
+    yr_object_copy(array->prototype_item, &result);
 
     if (result != NULL)
       yr_object_array_set_item(object, result, index);
@@ -554,7 +681,7 @@ int yr_object_array_set_item(
   {
     count = max(64, (index + 1) * 2);
 
-    array->items = yr_malloc(
+    array->items = (YR_ARRAY_ITEMS*) yr_malloc(
         sizeof(YR_ARRAY_ITEMS) + count * sizeof(YR_OBJECT*));
 
     if (array->items == NULL)
@@ -567,7 +694,7 @@ int yr_object_array_set_item(
   else if (index >= array->items->count)
   {
     count = array->items->count * 2;
-    array->items = yr_realloc(
+    array->items = (YR_ARRAY_ITEMS*) yr_realloc(
         array->items,
         sizeof(YR_ARRAY_ITEMS) + count * sizeof(YR_OBJECT*));
 
@@ -587,9 +714,101 @@ int yr_object_array_set_item(
 }
 
 
+YR_OBJECT* yr_object_dict_get_item(
+    YR_OBJECT* object,
+    int flags,
+    const char* key)
+{
+  YR_OBJECT* result = NULL;
+  YR_OBJECT_DICTIONARY* dict;
+
+  assert(object->type == OBJECT_TYPE_DICTIONARY);
+
+  dict = (YR_OBJECT_DICTIONARY*) object;
+
+  if (dict->items != NULL)
+  {
+    for (int i = 0; i < dict->items->used; i++)
+    {
+      if (strcmp(dict->items->objects[i].key, key) == 0)
+        result = dict->items->objects[i].obj;
+    }
+  }
+
+  if (result == NULL && flags & OBJECT_CREATE)
+  {
+    yr_object_copy(dict->prototype_item, &result);
+
+    if (result != NULL)
+      yr_object_dict_set_item(object, result, key);
+  }
+
+  return result;
+}
+
+
+int yr_object_dict_set_item(
+    YR_OBJECT* object,
+    YR_OBJECT* item,
+    const char* key)
+{
+  YR_OBJECT_DICTIONARY* dict;
+
+  int i;
+  int count;
+
+  assert(object->type == OBJECT_TYPE_DICTIONARY);
+
+  dict = ((YR_OBJECT_DICTIONARY*) object);
+
+  if (dict->items == NULL)
+  {
+    count = 64;
+
+    dict->items = (YR_DICTIONARY_ITEMS*) yr_malloc(
+        sizeof(YR_DICTIONARY_ITEMS) + count * sizeof(dict->items->objects[0]));
+
+    if (dict->items == NULL)
+      return ERROR_INSUFICIENT_MEMORY;
+
+    memset(dict->items->objects, 0, count * sizeof(dict->items->objects[0]));
+
+    dict->items->free = count;
+    dict->items->used = 0;
+  }
+  else if (dict->items->free == 0)
+  {
+    count = dict->items->used * 2;
+    dict->items = (YR_DICTIONARY_ITEMS*) yr_realloc(
+        dict->items,
+        sizeof(YR_DICTIONARY_ITEMS) + count * sizeof(dict->items->objects[0]));
+
+    if (dict->items == NULL)
+      return ERROR_INSUFICIENT_MEMORY;
+
+    for (i = dict->items->used; i < count; i++)
+    {
+      dict->items->objects[i].key = NULL;
+      dict->items->objects[i].obj = NULL;
+    }
+
+    dict->items->free = dict->items->used;
+  }
+
+  item->parent = object;
+
+  dict->items->objects[dict->items->used].key = yr_strdup(key);
+  dict->items->objects[dict->items->used].obj = item;
+
+  dict->items->used++;
+
+  return ERROR_SUCCESS;
+}
+
+
 int64_t yr_object_get_integer(
     YR_OBJECT* object,
-    char* field,
+    const char* field,
     ...)
 {
   YR_OBJECT* integer_obj;
@@ -614,9 +833,9 @@ int64_t yr_object_get_integer(
 }
 
 
-char* yr_object_get_string(
+SIZED_STRING* yr_object_get_string(
     YR_OBJECT* object,
-    char* field,
+    const char* field,
     ...)
 {
   YR_OBJECT* string_obj;
@@ -644,7 +863,7 @@ char* yr_object_get_string(
 void yr_object_set_integer(
     int64_t value,
     YR_OBJECT* object,
-    char* field,
+    const char* field,
     ...)
 {
   YR_OBJECT* integer_obj;
@@ -668,9 +887,10 @@ void yr_object_set_integer(
 
 
 void yr_object_set_string(
-    char* value,
+    const char* value,
+    size_t len,
     YR_OBJECT* object,
-    char* field,
+    const char* field,
     ...)
 {
   YR_OBJECT_STRING* string_obj;
@@ -692,7 +912,18 @@ void yr_object_set_string(
   if (string_obj->value != NULL)
     yr_free(string_obj->value);
 
-  string_obj->value = yr_strdup(value);
+  if (value != NULL)
+  {
+    string_obj->value = (SIZED_STRING*) yr_malloc(len + sizeof(SIZED_STRING));
+    string_obj->value->length = len;
+    string_obj->value->flags = 0;
+
+    memcpy(string_obj->value->c_string, value, len);
+  }
+  else
+  {
+    string_obj->value = NULL;
+  }
 }
 
 
@@ -706,3 +937,88 @@ YR_OBJECT* yr_object_get_root(
 
   return o;
 }
+
+void yr_object_print_data(
+    YR_OBJECT* object,
+    int indent)
+{
+  YR_DICTIONARY_ITEMS* dict_items;
+  YR_ARRAY_ITEMS* array_items;
+  YR_STRUCTURE_MEMBER* member;
+
+  char indent_spaces[32];
+
+  indent = min(indent, sizeof(indent_spaces));
+
+  memset(indent_spaces, '\t', indent);
+  indent_spaces[indent] = '\0';
+
+  switch(object->type)
+  {
+    case OBJECT_TYPE_INTEGER:
+      if (((YR_OBJECT_INTEGER*) object)->value != UNDEFINED)
+        printf(
+            "%s%s = %" PRIu64 "\n",
+            indent_spaces,
+            object->identifier,
+            ((YR_OBJECT_INTEGER*) object)->value);
+      break;
+
+    case OBJECT_TYPE_STRING:
+      if (((YR_OBJECT_STRING*) object)->value != NULL)
+        printf(
+            "%s%s = \"%s\"\n",
+            indent_spaces,
+            object->identifier,
+            ((YR_OBJECT_STRING*) object)->value->c_string);
+      break;
+
+    case OBJECT_TYPE_STRUCTURE:
+      printf(
+          "%s%s\n",
+          indent_spaces,
+          object->identifier);
+
+      member = ((YR_OBJECT_STRUCTURE*) object)->members;
+
+      while (member != NULL)
+      {
+        yr_object_print_data(member->object, indent + 1);
+        member = member->next;
+      }
+
+      break;
+
+    case OBJECT_TYPE_ARRAY:
+      array_items = ((YR_OBJECT_ARRAY*) object)->items;
+
+      if (array_items != NULL)
+      {
+        for (int i = 0; i < array_items->count; i++)
+        {
+          if (array_items->objects[i] != NULL)
+          {
+            printf("%s[%d]\n", indent_spaces, i);
+            yr_object_print_data(array_items->objects[i], indent + 1);
+          }
+        }
+      }
+
+      break;
+
+    case OBJECT_TYPE_DICTIONARY:
+      dict_items = ((YR_OBJECT_DICTIONARY*) object)->items;
+
+      if (dict_items != NULL)
+      {
+        printf("%s%s\n", indent_spaces, object->identifier);
+
+        for (int i = 0; i < dict_items->used; i++)
+        {
+          printf("%s\t%s\n", indent_spaces, dict_items->objects[i].key);
+          yr_object_print_data(dict_items->objects[i].obj, indent + 1);
+        }
+      }
+      break;
+  }
+}
