@@ -1,4 +1,4 @@
-    /*
+/*
 Copyright (c) 2013. The YARA Authors. All Rights Reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
@@ -23,6 +23,7 @@ limitations under the License.
 #include <yara/error.h>
 #include <yara/exec.h>
 #include <yara/object.h>
+#include <yara/strutils.h>
 #include <yara/utils.h>
 #include <yara/modules.h>
 #include <yara/parser.h>
@@ -141,11 +142,12 @@ int yr_parser_emit_pushes_for_strings(
             NULL);
 
         string->g_flags |= STRING_GFLAGS_REFERENCED;
+        string->g_flags &= ~STRING_GFLAGS_FIXED_OFFSET;
         matching++;
       }
     }
 
-    string = yr_arena_next_address(
+    string = (YR_STRING*) yr_arena_next_address(
         compiler->strings_arena,
         string,
         sizeof(YR_STRING));
@@ -166,51 +168,21 @@ int yr_parser_check_types(
     YR_OBJECT_FUNCTION* function,
     const char* actual_args_fmt)
 {
-  int i;
-
-  char message[MAX_COMPILER_ERROR_EXTRA_INFO];
-
-  const char* expected = function->arguments_fmt;
-  const char* actual = actual_args_fmt;
-
-  i = 0;
-
-  while (*expected != '\0' || *actual != '\0')
+  for (int i = 0; i < MAX_OVERLOADED_FUNCTIONS; i++)
   {
-    i++;
-
-    if (*expected != *actual)
-    {
-      if (*expected == '\0' || *actual == '\0')
-      {
-        snprintf(
-            message,
-            sizeof(message),
-            "wrong number of arguments for \"%s\"",
-            function->identifier);
-
-        compiler->last_result = ERROR_WRONG_NUMBER_OF_ARGUMENTS;
-      }
-      else
-      {
-        snprintf(
-            message,
-            sizeof(message),
-            "wrong type for argument %i of \"%s\"",
-            i,
-            function->identifier);
-
-        compiler->last_result = ERROR_WRONG_TYPE;
-      }
-
-      yr_compiler_set_error_extra_info(compiler, message);
+    if (function->prototypes[i].arguments_fmt == NULL)
       break;
-    }
 
-    expected++;
-    actual++;
+    if (strcmp(function->prototypes[i].arguments_fmt, actual_args_fmt) == 0)
+    {
+      compiler->last_result = ERROR_SUCCESS;
+      return compiler->last_result;
+    }
   }
 
+  yr_compiler_set_error_extra_info(compiler, function->identifier);
+  compiler->last_result = ERROR_WRONG_ARGUMENTS;
+
   return compiler->last_result;
 }
 
@@ -237,7 +209,7 @@ YR_STRING* yr_parser_lookup_string(
       return string;
     }
 
-    string = yr_arena_next_address(
+    string = (YR_STRING*) yr_arena_next_address(
         compiler->strings_arena,
         string,
         sizeof(YR_STRING));
@@ -275,12 +247,10 @@ int _yr_parser_write_string(
     SIZED_STRING* str,
     RE* re,
     YR_STRING** string,
-    int* min_atom_length)
+    int* min_atom_quality)
 {
   SIZED_STRING* literal_string;
   YR_AC_MATCH* new_match;
-
-  YR_ATOM_LIST_ITEM* atom;
   YR_ATOM_LIST_ITEM* atom_list = NULL;
 
   int result;
@@ -328,6 +298,7 @@ int _yr_parser_write_string(
 
   (*string)->g_flags = flags;
   (*string)->chained_to = NULL;
+  (*string)->fixed_offset = UNDEFINED;
 
   #ifdef PROFILING_ENABLED
   (*string)->clock_ticks = 0;
@@ -347,7 +318,7 @@ int _yr_parser_write_string(
         compiler->sz_arena,
         literal_string->c_string,
         literal_string->length,
-        (void*) &(*string)->string);
+        (void**) &(*string)->string);
 
     if (result == ERROR_SUCCESS)
     {
@@ -402,19 +373,7 @@ int _yr_parser_write_string(
     }
   }
 
-  atom = atom_list;
-
-  if (atom != NULL)
-    *min_atom_length = MAX_ATOM_LENGTH;
-  else
-    *min_atom_length = 0;
-
-  while (atom != NULL)
-  {
-    if (atom->atom_length < *min_atom_length)
-      *min_atom_length = atom->atom_length;
-    atom = atom->next;
-  }
+  *min_atom_quality = yr_atoms_min_quality(atom_list);
 
   if (flags & STRING_GFLAGS_LITERAL)
   {
@@ -423,7 +382,7 @@ int _yr_parser_write_string(
     else
       max_string_len = (*string)->length;
 
-    if (max_string_len == *min_atom_length)
+    if (max_string_len <= MAX_ATOM_LENGTH)
       (*string)->g_flags |= STRING_GFLAGS_FITS_IN_ATOM;
   }
 
@@ -446,8 +405,8 @@ YR_STRING* yr_parser_reduce_string_declaration(
     const char* identifier,
     SIZED_STRING* str)
 {
-  int min_atom_length;
-  int min_atom_length_aux;
+  int min_atom_quality;
+  int min_atom_quality_aux;
   int re_flags = 0;
 
   int32_t min_gap;
@@ -488,6 +447,14 @@ YR_STRING* yr_parser_reduce_string_declaration(
 
   string_flags |= STRING_GFLAGS_SINGLE_MATCH;
 
+  // The STRING_GFLAGS_FIXED_OFFSET indicates that the string doesn't
+  // need to be searched all over the file because the user is using the
+  // "at" operator. The string must be searched at a fixed offset in the
+  // file. All strings are marked STRING_GFLAGS_FIXED_OFFSET initially,
+  // and unmarked later if required.
+
+  string_flags |= STRING_GFLAGS_FIXED_OFFSET;
+
   if (string_flags & STRING_GFLAGS_HEXADECIMAL ||
       string_flags & STRING_GFLAGS_REGEXP)
   {
@@ -518,6 +485,17 @@ YR_STRING* yr_parser_reduce_string_declaration(
     if (re->flags & RE_FLAGS_FAST_HEX_REGEXP)
       string_flags |= STRING_GFLAGS_FAST_HEX_REGEXP;
 
+    if (yr_re_contains_dot_star(re))
+    {
+      snprintf(
+        message,
+        sizeof(message),
+        "%s contains .*, consider using .{N} with a reasonable value for N",
+        identifier);
+
+        yywarning(yyscanner, message);
+    }
+
     compiler->last_result = yr_re_split_at_chaining_point(
         re, &re, &remainder_re, &min_gap, &max_gap);
 
@@ -531,7 +509,7 @@ YR_STRING* yr_parser_reduce_string_declaration(
         NULL,
         re,
         &string,
-        &min_atom_length);
+        &min_atom_quality);
 
     if (compiler->last_result != ERROR_SUCCESS)
       goto _exit;
@@ -570,19 +548,25 @@ YR_STRING* yr_parser_reduce_string_declaration(
           NULL,
           re,
           &aux_string,
-          &min_atom_length_aux);
+          &min_atom_quality_aux);
 
       if (compiler->last_result != ERROR_SUCCESS)
         goto _exit;
 
-      if (min_atom_length_aux < min_atom_length)
-        min_atom_length = min_atom_length_aux;
+      if (min_atom_quality_aux < min_atom_quality)
+        min_atom_quality = min_atom_quality_aux;
 
       aux_string->g_flags |= STRING_GFLAGS_CHAIN_PART;
       aux_string->chain_gap_min = min_gap;
       aux_string->chain_gap_max = max_gap;
 
       prev_string->chained_to = aux_string;
+
+      // prev_string is now chained to aux_string, an string chained
+      // to another one can't have a fixed offset, only the head of the
+      // string chain can have a fixed offset.
+
+      prev_string->g_flags &= ~STRING_GFLAGS_FIXED_OFFSET;
     }
   }
   else
@@ -594,20 +578,20 @@ YR_STRING* yr_parser_reduce_string_declaration(
         str,
         NULL,
         &string,
-        &min_atom_length);
+        &min_atom_quality);
 
     if (compiler->last_result != ERROR_SUCCESS)
       goto _exit;
   }
 
-  if (min_atom_length < 2 && compiler->callback != NULL)
+  if (min_atom_quality < 3 && compiler->callback != NULL)
   {
     snprintf(
         message,
         sizeof(message),
         "%s is slowing down scanning%s",
         string->identifier,
-        min_atom_length == 0 ? " (critical!)" : "");
+        min_atom_quality < 2 ? " (critical!)" : "");
 
     yywarning(yyscanner, message);
   }
@@ -650,7 +634,7 @@ int yr_parser_reduce_rule_declaration(
     // appropriate error.
 
     yr_compiler_set_error_extra_info(compiler, identifier);
-    compiler->last_result = ERROR_DUPLICATE_IDENTIFIER;
+    compiler->last_result = ERROR_DUPLICATED_IDENTIFIER;
     return compiler->last_result;
   }
 
@@ -672,7 +656,7 @@ int yr_parser_reduce_rule_declaration(
       break;
     }
 
-    string = yr_arena_next_address(
+    string = (YR_STRING*) yr_arena_next_address(
         compiler->strings_arena,
         string,
         sizeof(YR_STRING));
@@ -729,14 +713,15 @@ int yr_parser_reduce_rule_declaration(
 int yr_parser_reduce_string_identifier(
     yyscan_t yyscanner,
     const char* identifier,
-    int8_t instruction)
+    int8_t instruction,
+    uint64_t at_offset)
 {
   YR_STRING* string;
   YR_COMPILER* compiler = yyget_extra(yyscanner);
 
-  if (strcmp(identifier, "$") == 0)
+  if (strcmp(identifier, "$") == 0) // is an anonymous string ?
   {
-    if (compiler->loop_for_of_mem_offset >= 0)
+    if (compiler->loop_for_of_mem_offset >= 0) // inside a loop ?
     {
       yr_parser_emit_with_arg(
           yyscanner,
@@ -746,22 +731,41 @@ int yr_parser_reduce_string_identifier(
 
       yr_parser_emit(yyscanner, instruction, NULL);
 
-      if (instruction != OP_STR_FOUND)
-      {
-        string = compiler->current_rule_strings;
+      string = compiler->current_rule_strings;
 
-        while(!STRING_IS_NULL(string))
-        {
+      while(!STRING_IS_NULL(string))
+      {
+        if (instruction != OP_FOUND)
           string->g_flags &= ~STRING_GFLAGS_SINGLE_MATCH;
-          string = yr_arena_next_address(
-              compiler->strings_arena,
-              string,
-              sizeof(YR_STRING));
+
+        if (instruction == OP_FOUND_AT)
+        {
+          // Avoid overwriting any previous fixed offset
+
+          if (string->fixed_offset == UNDEFINED)
+            string->fixed_offset = at_offset;
+
+          // If a previous fixed offset was different, disable
+          // the STRING_GFLAGS_FIXED_OFFSET flag because we only
+          // have room to store a single fixed offset value
+
+          if (string->fixed_offset != at_offset)
+            string->g_flags &= ~STRING_GFLAGS_FIXED_OFFSET;
         }
+        else
+        {
+          string->g_flags &= ~STRING_GFLAGS_FIXED_OFFSET;
+        }
+
+        string = (YR_STRING*) yr_arena_next_address(
+            compiler->strings_arena,
+            string,
+            sizeof(YR_STRING));
       }
     }
     else
     {
+      // Anonymous strings not allowed outside of a loop
       compiler->last_result = ERROR_MISPLACED_ANONYMOUS_STRING;
     }
   }
@@ -777,9 +781,31 @@ int yr_parser_reduce_string_identifier(
           PTR_TO_UINT64(string),
           NULL);
 
-      if (instruction != OP_STR_FOUND)
+      if (instruction != OP_FOUND)
         string->g_flags &= ~STRING_GFLAGS_SINGLE_MATCH;
 
+      if (instruction == OP_FOUND_AT)
+      {
+        // Avoid overwriting any previous fixed offset
+
+        if (string->fixed_offset == UNDEFINED)
+          string->fixed_offset = at_offset;
+
+        // If a previous fixed offset was different, disable
+        // the STRING_GFLAGS_FIXED_OFFSET flag because we only
+        // have room to store a single fixed offset value
+
+        if (string->fixed_offset == UNDEFINED ||
+            string->fixed_offset != at_offset)
+        {
+          string->g_flags &= ~STRING_GFLAGS_FIXED_OFFSET;
+        }
+      }
+      else
+      {
+        string->g_flags &= ~STRING_GFLAGS_FIXED_OFFSET;
+      }
+
       yr_parser_emit(yyscanner, instruction, NULL);
 
       string->g_flags |= STRING_GFLAGS_REFERENCED;
@@ -846,7 +872,7 @@ int yr_parser_reduce_import(
 
   char* name;
 
-  module_structure = yr_hash_table_lookup(
+  module_structure = (YR_OBJECT*) yr_hash_table_lookup(
       compiler->objects_table,
       module_name->c_string,
       compiler->current_namespace->name);
