@@ -142,9 +142,9 @@ int yr_re_finalize_thread(void)
   RE_THREAD_STORAGE* storage;
 
   #ifdef _WIN32
-  storage = TlsGetValue(thread_storage_key);
+  storage = (RE_THREAD_STORAGE*) TlsGetValue(thread_storage_key);
   #else
-  storage = pthread_getspecific(thread_storage_key);
+  storage = (RE_THREAD_STORAGE*) pthread_getspecific(thread_storage_key);
   #endif
 
   if (storage != NULL)
@@ -176,7 +176,7 @@ RE_NODE* yr_re_node_create(
     RE_NODE* left,
     RE_NODE* right)
 {
-  RE_NODE* result = yr_malloc(sizeof(RE_NODE));
+  RE_NODE* result = (RE_NODE*) yr_malloc(sizeof(RE_NODE));
 
   if (result != NULL)
   {
@@ -386,7 +386,7 @@ SIZED_STRING* yr_re_extract_literal(
     node = node->left;
   }
 
-  string = yr_malloc(sizeof(SIZED_STRING) + length);
+  string = (SIZED_STRING*) yr_malloc(sizeof(SIZED_STRING) + length);
 
   if (string == NULL)
     return NULL;
@@ -416,6 +416,29 @@ SIZED_STRING* yr_re_extract_literal(
 }
 
 
+int _yr_re_node_contains_dot_star(
+    RE_NODE* re_node)
+{
+  if (re_node->type == RE_NODE_STAR && re_node->left->type == RE_NODE_ANY)
+    return TRUE;
+
+  if (re_node->left != NULL && _yr_re_node_contains_dot_star(re_node->left))
+    return TRUE;
+
+  if (re_node->right != NULL && _yr_re_node_contains_dot_star(re_node->right))
+    return TRUE;
+
+  return FALSE;
+}
+
+
+int yr_re_contains_dot_star(
+    RE* re)
+{
+  return _yr_re_node_contains_dot_star(re->root_node);
+}
+
+
 int yr_re_split_at_chaining_point(
     RE* re,
     RE** result_re,
@@ -1105,14 +1128,14 @@ int _yr_re_alloc_storage(
     RE_THREAD_STORAGE** storage)
 {
   #ifdef _WIN32
-  *storage = TlsGetValue(thread_storage_key);
+  *storage = (RE_THREAD_STORAGE*) TlsGetValue(thread_storage_key);
   #else
-  *storage = pthread_getspecific(thread_storage_key);
+  *storage = (RE_THREAD_STORAGE*) pthread_getspecific(thread_storage_key);
   #endif
 
   if (*storage == NULL)
   {
-    *storage = yr_malloc(sizeof(RE_THREAD_STORAGE));
+    *storage = (RE_THREAD_STORAGE*) yr_malloc(sizeof(RE_THREAD_STORAGE));
 
     if (*storage == NULL)
       return ERROR_INSUFICIENT_MEMORY;
@@ -1145,7 +1168,7 @@ RE_FIBER* _yr_re_fiber_create(
   }
   else
   {
-    fiber = yr_malloc(sizeof(RE_FIBER));
+    fiber = (RE_FIBER*) yr_malloc(sizeof(RE_FIBER));
   }
 
   if (fiber != NULL)
