@@ -17,7 +17,7 @@
 from distutils.core import setup, Extension
 
 setup(name='yara-python',
-      version='3.1',
+      version='3.2.0',
       author='Victor M. Alvarez',
       author_email='plusvic@gmail.com;vmalvarez@virustotal.com',
       ext_modules=[Extension(
@@ -25,5 +25,5 @@ setup(name='yara-python',
         sources=['yara-python.c'],
         include_dirs=['../windows/include', '../libyara/include'],
         extra_objects=[
-          '../windows/libyara/Release/libyara64.lib']
+          'user32.lib', 'gdi32.lib', '../windows/libyara/Release/libyara64.lib']
         )])
