@@ -274,7 +274,8 @@ class TestYara(unittest.TestCase):
         self.assertTrueRules([
             'rule test { condition: true }',
             'rule test { condition: true or false }',
-            'rule test { condition: true and true }'
+            'rule test { condition: true and true }',
+            'rule test { condition: 0x1 and 0x2}',
         ])
 
         self.assertFalseRules([
@@ -319,6 +320,7 @@ class TestYara(unittest.TestCase):
 
         self.assertTrueRules([
             'rule test { strings: $a = "a" condition: $a }',
+            'rule test { strings: $a = "ab" condition: $a }',
             'rule test { strings: $a = "abc" condition: $a }',
             'rule test { strings: $a = "xyz" condition: $a }',
             'rule test { strings: $a = "abc" nocase fullword condition: $a }',
@@ -326,11 +328,21 @@ class TestYara(unittest.TestCase):
             'rule test { strings: $a = "abc" fullword condition: $a }',
         ], "---- abc ---- xyz")
 
+        self.assertFalseRules([
+            'rule test { strings: $a = "a" fullword condition: $a }',
+            'rule test { strings: $a = "ab" fullword condition: $a }',
+            'rule test { strings: $a = "abc" wide fullword condition: $a }',
+        ], "---- abc ---- xyz")
+
         self.assertTrueRules([
             'rule test { strings: $a = "a" wide condition: $a }',
+            'rule test { strings: $a = "a" wide ascii condition: $a }',
+            'rule test { strings: $a = "ab" wide condition: $a }',
+            'rule test { strings: $a = "ab" wide ascii condition: $a }',
             'rule test { strings: $a = "abc" wide condition: $a }',
             'rule test { strings: $a = "abc" wide nocase fullword condition: $a }',
             'rule test { strings: $a = "aBc" wide nocase condition: $a }',
+            'rule test { strings: $a = "aBc" wide ascii nocase condition: $a }',
             'rule test { strings: $a = "---xyz" wide nocase condition: $a }'
         ], "---- a\x00b\x00c\x00 -\x00-\x00-\x00-\x00x\x00y\x00z\x00")
 
@@ -350,6 +362,14 @@ class TestYara(unittest.TestCase):
             'rule test { strings: $a = "abc" fullword condition: $a }',
         ], "abcx")
 
+        self.assertFalseRules([
+            'rule test { strings: $a = "abc" ascii wide fullword condition: $a }',
+        ], "abcx")
+
+        self.assertTrueRules([
+            'rule test { strings: $a = "abc" ascii wide fullword condition: $a }',
+        ], "a\x00abc")
+
         self.assertTrueRules([
             'rule test { strings: $a = "abc" wide fullword condition: $a }',
         ], "a\x00b\x00c\x00")
@@ -358,6 +378,10 @@ class TestYara(unittest.TestCase):
             'rule test { strings: $a = "abc" wide fullword condition: $a }',
         ], "x\x00a\x00b\x00c\x00x\x00")
 
+        self.assertFalseRules([
+            'rule test { strings: $a = "ab" wide fullword condition: $a }',
+        ], "x\x00a\x00b\x00")
+
         self.assertFalseRules([
             'rule test { strings: $a = "abc" wide fullword condition: $a }',
         ], "x\x00a\x00b\x00c\x00")
@@ -703,7 +727,11 @@ class TestYara(unittest.TestCase):
             'import "tests" rule test { condition: tests.integer_array[1] == 1}',
             'import "tests" rule test { condition: tests.string_array[0] == "foo"}',
             'import "tests" rule test { condition: tests.string_array[2] == "baz"}',
-            'import "tests" rule test { condition: tests.sum(1,1) == 2}',
+            'import "tests" rule test { condition: tests.string_dict["foo"] == "foo"}',
+            'import "tests" rule test { condition: tests.string_dict["bar"] == "bar"}',
+            'import "tests" rule test { condition: tests.sum(1,2) == 3}',
+            'import "tests" rule test { condition: tests.sum(1,2,3) == 6}',
+            'import "tests" rule test { condition: tests.length("dummy") == 5}',
           ])
 
         self.assertFalseRules([
@@ -711,6 +739,17 @@ class TestYara(unittest.TestCase):
             'import "tests" rule test { condition: tests.sum(1,1) == 3}',
           ])
 
+    def testIntegerFunctions(self):
+
+        self.assertTrueRules([
+            'rule test { condition: uint8(0) == 0xAA}',
+            'rule test { condition: uint16(0) == 0xBBAA}',
+            'rule test { condition: uint32(0) == 0xDDCCBBAA}',
+            'rule test { condition: uint8be(0) == 0xAA}',
+            'rule test { condition: uint16be(0) == 0xAABB}',
+            'rule test { condition: uint32be(0) == 0xAABBCCDD}',
+          ], '\xAA\xBB\xCC\xDD')
+
 
 if __name__ == "__main__":
     unittest.main()
